/* Generated automatically by the program `genattr-common'
   from the machine description file `md'.  */

#ifndef GCC_INSN_ATTR_COMMON_H
#define GCC_INSN_ATTR_COMMON_H

enum attr_type {TYPE_UNKNOWN, TYPE_JUMP, TYPE_CALL, TYPE_LOAD, TYPE_STORE, TYPE_MOVE, TYPE_ARITH, TYPE_MULTI, TYPE_NOP, TYPE_FARITH, TYPE_FMADD, TYPE_FCONV, TYPE_FLOAD, TYPE_FSTORE, TYPE_MUL16, TYPE_MUL32, TYPE_DIV32, TYPE_MAC16, TYPE_RSR, TYPE_WSR, TYPE_ENTRY, TYPE_TRAP};
enum attr_mode {MODE_UNKNOWN, MODE_NONE, MODE_QI, MODE_HI, MODE_SI, MODE_DI, MODE_SF, MODE_DF, MODE_BL};
#define INSN_SCHEDULING
#define DELAY_SLOTS 0

#endif /* GCC_INSN_ATTR_COMMON_H */
